<?php

/*
Code responsible for loading required parts of the theme
Written by Chris Jean for iThemes.com
Version 1.6.0

Version History
	1.3.0 - 2011-08-04 - Chris Jean
		Added feature to add Favicons to Dashboard
	1.4.0 - 2011-10-06 - Chris Jean
		Optimized some of the code to be faster
		Added handler for builder-admin-bar theme support
		Added hooks previously in lib/layout-engine/init.php:
			builder_template_show_not_found
			builder_comments_popup_link
			builder_module_filter_css_prefix
			img_caption_shortcode
			post_gallery
	1.5.0 - 2011-10-19 - Chris Jean
		Added loader for admin-functions.php
		Removed all references to TEMPLATEPATH
	1.6.0 - 2011-12-05 - Chris Jean
		Moved loader for widgets/init.php above theme-settings/init.php to
			allow filtering of the default values
		Hooked the builder_filter_wp_title function to the wp_title filter
*/


do_action( 'builder_start' );

// Set theme-specific global variables
$GLOBALS['wp_theme_name']          = 'Builder';
$GLOBALS['theme_index']            = 'it-builder';
$GLOBALS['theme_menu_var']         = 'ithemes-builder-theme';
$GLOBALS['wp_theme_page_name']     = 'ithemes-builder-theme';
$GLOBALS['builder_start_here_url'] = 'http://ithemes.com/start-here-tuts/?site=' . get_option( 'siteurl' );


require_once( dirname( dirname( __FILE__ ) ) . '/classes/load.php' );
require_once( dirname( __FILE__ ) . '/functions.php' );
require_once( dirname( __FILE__ ) . '/compat.php' );

if ( is_admin() )
	require_once( dirname( __FILE__ ) . '/admin-functions.php' );


// Set the memory_limit to be at least 64M
// This is to help bypass out of memory errors that happen with WordPress 3.0:
// http://core.trac.wordpress.org/ticket/14889
builder_set_minimum_memory_limit( '64M' );


function it_builder_load_theme_features() {
	global $wp_version;
	
	$path = builder_template_directory();
	
	
	it_classes_load( 'it-cache.php' );
	
	require_once( $path . '/lib/import-export/init.php' );
	require_once( $path . '/lib/widgets/init.php' );
	require_once( $path . '/lib/theme-settings/init.php' );
	require_once( $path . '/lib/layout-engine/init.php' );
	
	
	$file_cache = ( builder_theme_supports( 'builder-file-cache' ) ) ? true : false;
	
	$GLOBALS['builder_cache'] =& new ITCache( 'builder-core', array( 'enable_file_cache' => $file_cache ) );
	$GLOBALS['builder_cache']->add_content_type( 'javascript-footer', 'javascript-footer.js', 'text/javascript', array( 'async_load' => true ) );
	$GLOBALS['builder_cache']->add_content_filter( 'javascript', 'builder_filter_javascript_content' );
	$GLOBALS['builder_cache']->add_content_filter( 'javascript-footer', 'builder_filter_javascript_footer_content' );
	$GLOBALS['builder_cache']->add_content_filter( 'css', 'builder_filter_css_content' );
	
	
	// Add support for builder-3.0 if the Builder core theme is the active theme
	if ( builder_parent_is_active() )
		add_theme_support( 'builder-3.0' );
	
	
	if ( builder_theme_supports( 'builder-my-theme-menu' ) )
		require_once( $path . '/lib/tutorials/init.php' );
	
	// Compatibility check for pre-3.0 automatic-feed-links support
	if ( version_compare( $wp_version, '2.9.7', '<=' ) && builder_theme_supports( 'automatic-feed-links' ) && function_exists( 'automatic_feed_links' ) )
		automatic_feed_links();
	
	if ( builder_theme_supports( 'builder-extensions' ) )
		require_once( $path . '/lib/extensions/init.php' );
	
	if ( builder_theme_supports( 'builder-admin-bar' ) )
		require_once( $path . '/lib/admin-bar/init.php' );
	
	if ( builder_theme_supports( 'builder-plugin-features' ) )
		require_once( $path . '/lib/plugin-features/init.php' );
	
	if ( builder_theme_supports( 'builder-3.0' ) ) {
		add_theme_support( 'loop-standard' );
		require_once( $path . '/lib/loop-standard/functions.php' );
	}
	
	
	if ( 'on' == builder_get_theme_setting( 'dashboard_favicon' ) )
		add_action( 'admin_enqueue_scripts', 'builder_add_favicon', 0 );
	
	
	do_action( 'builder_theme_features_loaded' );
}
add_action( 'it_libraries_loaded', 'it_builder_load_theme_features', -10 );


// Now the text widget supports shortcodes
add_filter( 'widget_text', 'do_shortcode' );


add_action( 'builder_add_title', 'builder_add_title' );
add_action( 'admin_print_styles', 'builder_add_global_admin_styles' );
add_action( 'comment_form_comments_closed', 'builder_get_closed_comments_message' );
add_action( 'builder_template_show_not_found', 'builder_template_show_not_found' );
add_action( 'builder_comments_popup_link', 'builder_comments_popup_link', 10, 6 );


add_filter( 'builder_filter_favicon_url', 'builder_filter_favicon_url' );
add_filter( 'it_filter_theme_menu_var', 'it_set_theme_menu_var' );
add_filter( 'it_storage_filter_theme_index', 'it_set_theme_index' );
add_filter( 'it_tutorials_top_menu_icon', 'filter_it_tutorials_top_menu_icon' );
add_filter( 'it_tutorials_filter_url', 'builder_set_start_here_url' );
add_filter( 'admin_body_class', 'builder_filter_admin_body_classes' );
add_filter( 'builder_module_filter_css_prefix', 'builder_module_filter_css_prefix', 0 );
add_filter( 'img_caption_shortcode', 'builder_custom_caption_shortcode', 10, 3 );
add_filter( 'post_gallery', 'builder_custom_post_gallery', 10, 2 );
add_filter( 'wp_title', 'builder_filter_wp_title', 10, 3 );
