<?php

/*
Written by Chris Jean for iThemes.com
Version 4.3.0

Version History
	3.3.0 - 2011-06-29 - Chris Jean
		Added a builder_debug query var check that allows for printing the rendering details w/o logging in
		Added support for the new Views data format
	3.4.0 - 2011-07-06 - Chris Jean
		Added support for legacy templates for child themes that don't have builder-3.0 support
	3.4.1 - 2011-08-04 - Chris Jean
		Changed legacy_templates to legacy-templates
	4.0.0 - 2011-10-06 - Chris Jean
		Split out the Layout selection process to the new BuilderLayoutSelector class
	4.1.0 - 2011-10-19 - Chris Jean
		Added check for builder-header-flush theme support
	4.2.0 - 2011-12-05 - Chris Jean
		Added support for proper Layout rendering with legacy WordPress templates
	4.2.1 - 2012-02-16 - Chris Jean
		Fixed bug where the *-module-style-outer-wrapper class was being applied twice
	4.3.0 - 2012-02-21 - Chris Jean
		Updated the generation of the classes for module outer wrappers to keep the classes for inner and
			outer wrappers the same
*/


if ( ! class_exists( 'BuilderLayoutEngine' ) ) {
	class BuilderLayoutEngine {
		var $_layout_id = false;
		var $_layout = false;
		var $_modules = array();
		var $_current_module = null;
		var $_current_area_width = null;
		var $_legacy_theme_template = false;
		
		
		function BuilderLayoutEngine() {
			$this->_modules = apply_filters( 'builder_get_modules', array() );
			
			
			add_action( 'get_header', array( &$this, 'identify_legacy_theme_template' ), -9999 );
			
			add_action( 'builder_layout_engine_identified_layout', array( &$this, 'layout_identified' ), 0, 2 );
			
			add_action( 'builder_layout_engine_render_layout', array( &$this, 'render' ), 10, 3 );
			
			add_action( 'builder_layout_engine_render_header', 'get_header', 10, 0 );
			add_action( 'builder_layout_engine_render_header', array( &$this, 'print_body_tag' ), 16 );
			
			add_action( 'builder_layout_engine_render_container', array( &$this, 'render_container' ) );
			add_action( 'builder_layout_engine_render_container_contents', array( &$this, 'render_container_contents' ) );
			
			add_action( 'builder_layout_engine_render_modules', array( &$this, 'render_modules' ) );
			add_action( 'builder_module_render', array( &$this, 'render_module' ) );
			add_action( 'builder_module_render_contents', array( &$this, 'render_module_contents' ) );
			add_action( 'builder_module_render_element_block', array( &$this, 'render_module_element_block' ) );
			add_action( 'builder_module_render_sidebar_block', array( &$this, 'render_module_sidebar_block' ), 10, 2 );
			add_action( 'builder_module_render_sidebar_block_contents', array( &$this, 'render_module_sidebar_block_contents' ), 10, 2 );
			
			add_action( 'builder_finish', array( &$this, 'render_finish' ) );
			
			add_action( 'builder_layout_engine_set_current_module', array( &$this, 'set_current_module' ) );
			add_action( 'builder_layout_engine_set_current_area_width', array( &$this, 'set_current_area_width' ) );
			
			
			add_filter( 'builder_layout_engine_get_current_module', array( &$this, 'get_current_module' ) );
			add_filter( 'builder_layout_engine_get_current_area_width', array( &$this, 'get_current_area_width' ) );
			
			add_filter( 'builder_filter_container_outer_wrapper_attributes', array( &$this, 'get_container_outer_wrapper_attributes' ) );
			add_filter( 'builder_get_container_width', array( &$this, 'get_container_width' ), 0 );
		}
		
		function identify_legacy_theme_template() {
			if ( did_action( 'builder_layout_engine_render' ) )
				return;
			
			
			$this->_legacy_theme_template_file = '';
			$backtrace = debug_backtrace();
			
			if ( isset( $backtrace[3] ) && isset( $backtrace[3]['file'] ) )
				$this->_legacy_theme_template_file = $backtrace[3]['file'];
			
			$this->_legacy_theme_template = true;
			ob_start();
			
			add_action( 'get_footer', array( $this, 'render_legacy_theme_template' ), -9999 );
		}
		
		function render_legacy_theme_template() {
			if ( isset( $this->_legacy_theme_template_content ) )
				return;
			
			$content = ob_get_contents();
			ob_end_clean();
			
			
			list( $head, $content ) = preg_split( '|<\s*/\s*head\s*>|i', $content, 2 );
			
			$this->_legacy_theme_template_head = "$head</head>\n";
			$this->_legacy_theme_template_content = $content;
			
			
			remove_action( 'get_header', array( &$this, 'identify_legacy_theme_template' ) );
			remove_action( 'builder_layout_engine_render_header', 'get_header' );
			remove_action( 'get_footer', array( $this, 'render_legacy_theme_template' ) );
			
			add_action( 'builder_layout_engine_render_header', array( $this, 'render_legacy_theme_template_head' ) );
			add_action( 'builder_layout_engine_render_content', array( $this, 'render_legacy_theme_template_content' ) );
			
			do_action( 'builder_layout_engine_render', basename( $this->_legacy_theme_template_file ) );
			
			exit;
		}
		
		function render_legacy_theme_template_head() {
			echo $this->_legacy_theme_template_head;
		}
		
		function render_legacy_theme_template_content() {
			echo $this->_legacy_theme_template_content;
		}
		
		function layout_identified( $layout_id, $layout_settings ) {
			$this->_layout_id = $layout_id;
			$this->_layout =& $layout_settings['layouts'][$layout_id];
		}
		
		function set_current_module( $module ) {
			$this->_current_module = $module;
		}
		
		function get_current_module( $module ) {
			return $this->_current_module;
		}
		
		function set_current_area_width( $width ) {
			$this->_current_area_width = $width;
		}
		
		function get_current_area_width( $width ) {
			return $this->_current_area_width;
		}
		
		function render( $view, $layout_id, $layout ) {
			$class_prefix = apply_filters( 'builder_module_filter_css_prefix', '' );
			
			$this->_layout_id = $layout_id;
			$this->_layout =& $layout;
			
			$render_data = array(
				'layout_id'    => $layout_id,
				'layout'       => $layout['description'],
				'class_prefix' => $class_prefix,
			);
			
			do_action( 'builder_layout_engine_render_header', $render_data );
			do_action( 'builder_layout_engine_render_container', $render_data );
			do_action( 'builder_finish', $render_data );
		}
		
		function render_finish() {
			echo "\n</body>\n</html>";
		}
		
		function render_container( $render_data ) {
			$this->_render_container_start();
			
			do_action( 'builder_layout_engine_render_container_contents', $render_data );
			
			$this->_render_container_end();
		}
		
		function render_container_contents( $render_data ) {
			do_action( 'builder_layout_engine_render_modules', $render_data );
		}
		
		function render_modules( $render_data ) {
			$class_prefix = $render_data['class_prefix'];
			
			
			$module_positions = array();
			$module_counts = array();
			$module_count = 0;
			$footer_position = 0;
			
			foreach ( (array) $this->_layout['modules'] as $module ) {
				$module_count++;
				
				if ( 'footer' === $module['module'] )
					$footer_position = $module_count;
				
				if ( ! isset( $module_counts[$module['module']] ) )
					$module_counts[$module['module']] = 0;
				$module_counts[$module['module']]++;
				
				$module_positions[$module_count] = $module['module'];
			}
			
			$module_use_count = array();
			$module_count = 0;
			
			$id = 1;
			$last_id = count( $this->_layout['modules'] );
			
			
			foreach ( (array) $this->_layout['modules'] as $module ) {
				if ( method_exists( $this->_modules[$module['module']], 'render' ) ) {
					if ( ! isset( $module_use_count[$module['module']] ) )
						$module_use_count[$module['module']] = 0;
					$module_use_count[$module['module']]++;
					
					$module_count++;
					
					if ( 1 === $last_id )
						$module_location = 'single';
					else if ( $module_count === $last_id )
						$module_location = 'bottom';
					else if ( 1 === $module_count )
						$module_location = 'top';
					else
						$module_location = 'middle';
					
					
					$module = array_merge( $module, $render_data );
					
					$module['id'] = $id;
					
					
					$module['inner_wrapper']['class'] = array(
						$class_prefix,
						"$class_prefix-{$module['module']}",
						"$class_prefix-$id",
						"$class_prefix-{$module['module']}-{$module_use_count[$module['module']]}",
						"$class_prefix-$module_location",
					);
					
					if ( $id === $last_id )
						$module['inner_wrapper']['class'][] = "$class_prefix-last";
					
					if ( $module_use_count[$module['module']] === $module_counts[$module['module']] )
						$module['inner_wrapper']['class'][] = "$class_prefix-{$module['module']}-last";
					
					if ( isset( $module_positions[$module_count + 1] ) )
						$module['inner_wrapper']['class'][] = "$class_prefix-before-{$module_positions[$module_count + 1]}";
					if ( ( $id === $last_id ) && ( 0 === $footer_position ) )
						$module['inner_wrapper']['class'][] = "$class_prefix-before-footer";
					if ( $module_count > 1 )
						$module['inner_wrapper']['class'][] = "$class_prefix-after-{$module_positions[$module_count - 1]}";
					
					if ( ! empty( $module['data']['style'] ) )
						$module['inner_wrapper']['class'][] = $module['data']['style'];
					else
						$module['inner_wrapper']['class'][] = 'default-module-style';
					
					
					foreach ( $module['inner_wrapper']['class'] as $class )
						$module['outer_wrapper']['class'][] = "$class-outer-wrapper";
					
					
					$module['inner_wrapper']['class'][] = 'clearfix';
					
					$module['inner_wrapper']['id'] = "$class_prefix-{$module['guid']}";
					
					
					do_action( 'builder_module_render', $module );
					
					$id++;
				}
			}
			
			if ( 0 === $footer_position ) {
				do_action( 'get_footer' );
				do_action( 'wp_footer' );
			}
		}
		
		function render_module( $fields ) {
			do_action( 'builder_layout_engine_set_current_module', $fields['module'] );
			
			do_action( "builder_module_render_{$fields['module']}", $fields );
			
			do_action( 'builder_layout_engine_set_current_module', null );
		}
		
		function render_module_contents( $fields ) {
			do_action( "builder_module_render_contents_{$fields['module']}", $fields );
		}
		
		function render_module_element_block( $fields ) {
			do_action( "builder_module_render_element_block_{$fields['module']}", $fields );
		}
		
		function render_module_sidebar_block( $fields, $side ) {
			do_action( "builder_module_render_sidebar_block_{$fields['module']}", $fields, $side );
		}
		
		function render_module_sidebar_block_contents( $fields, $side ) {
			do_action( "builder_module_render_sidebar_block_contents_{$fields['module']}", $fields, $side );
		}
		
		function print_body_tag( $render_data ) {
			if ( builder_theme_supports( 'builder-header-flush' ) && ( ! defined( 'BUILDER_DISABLE_FLUSH' ) || ( false === BUILDER_DISABLE_FLUSH ) ) )
				flush();
			
			$attributes = array(
				'id'		=> array(
					"builder-layout-{$this->_layout['guid']}",
				),
			);
			
			$attributes['class'] = get_body_class();
			
			$attributes = apply_filters( 'builder_filter_body_attributes', $attributes );
			
			ITUtility::print_open_tag( 'body', $attributes );
		}
			
		function get_container_width( $width ) {
			if ( 'custom' === $this->_layout['width'] )
				$width = intval( $this->_layout['custom_width'] );
			else
				$width = intval( $this->_layout['width'] );
			
			return $width;
		}
		
		function get_container_outer_wrapper_attributes( $options ) {
			$width = apply_filters( 'builder_get_container_width', 0 );
			
			$options['class'][] = 'builder-container-outer-wrapper';
			
			if ( ! empty( $width ) && ( $width > 0 ) ) {
				$options['style'][] = "width:{$width}px;";
				$options['style'][] = "max-width:{$width}px;";
			}
			
			return $options;
		}
		
		function _render_container_start() {
			$outer_wrapper_attributes = apply_filters( 'builder_filter_container_outer_wrapper_attributes', array() );
			
			$inner_wrapper_attributes = array( 'class' => array( 'builder-container' ), 'id' => "builder-container-{$this->_layout['guid']}" );
			$inner_wrapper_attributes = apply_filters( 'builder_filter_container_inner_wrapper_attributes', $inner_wrapper_attributes );
			
			ITUtility::print_open_tag( 'div', $outer_wrapper_attributes );
			ITUtility::print_open_tag( 'div', $inner_wrapper_attributes );
		}
		
		function _render_container_end() {
			echo "\n</div>\n</div>\n";
		}
	}
	
	new BuilderLayoutEngine();
}
